syntax = "proto2";

package commonroad;

import "obstacle.proto";
import "util.proto";

message EnvironmentObstacle {
    required uint32 environment_obstacle_id = 1;
    required ObstacleTypeEnum.ObstacleType obstacle_type = 2;
    required Shape obstacle_shape = 3;
}