# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lanelet.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='lanelet.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\rlanelet.proto\x12\ncommonroad\x1a\nutil.proto\"w\n\x0fLineMarkingEnum\"d\n\x0bLineMarking\x12\n\n\x06\x44\x41SHED\x10\x00\x12\t\n\x05SOLID\x10\x01\x12\x10\n\x0c\x42ROAD_DASHED\x10\x02\x12\x0f\n\x0b\x42ROAD_SOLID\x10\x03\x12\x0b\n\x07UNKNOWN\x10\x04\x12\x0e\n\nNO_MARKING\x10\x05\"6\n\x0e\x44rivingDirEnum\"$\n\nDrivingDir\x12\x08\n\x04SAME\x10\x00\x12\x0c\n\x08OPPOSITE\x10\x01\"\xbd\x02\n\x0fLaneletTypeEnum\"\xa9\x02\n\x0bLaneletType\x12\t\n\x05URBAN\x10\x00\x12\x0b\n\x07\x43OUNTRY\x10\x01\x12\x0b\n\x07HIGHWAY\x10\x02\x12\r\n\tDRIVE_WAY\x10\x03\x12\x15\n\x11MAIN_CARRIAGE_WAY\x10\x04\x12\x0f\n\x0b\x41\x43\x43\x45SS_RAMP\x10\x05\x12\r\n\tEXIT_RAMP\x10\x06\x12\x0c\n\x08SHOULDER\x10\x07\x12\x0c\n\x08\x42US_LANE\x10\x08\x12\x0c\n\x08\x42US_STOP\x10\t\x12\x10\n\x0c\x42ICYCLE_LANE\x10\n\x12\x0c\n\x08SIDEWALK\x10\x0b\x12\r\n\tCROSSWALK\x10\x0c\x12\x0e\n\nINTERSTATE\x10\r\x12\x10\n\x0cINTERSECTION\x10\x0e\x12\n\n\x06\x42ORDER\x10\x0f\x12\x0b\n\x07PARKING\x10\x10\x12\x0e\n\nRESTRICTED\x10\x11\x12\x0b\n\x07UNKNOWN\x10\x12\"\x9d\x01\n\x0cRoadUserEnum\"\x8c\x01\n\x08RoadUser\x12\x0b\n\x07VEHICLE\x10\x00\x12\x07\n\x03\x43\x41R\x10\x01\x12\t\n\x05TRUCK\x10\x02\x12\x07\n\x03\x42US\x10\x03\x12\x14\n\x10PRIORITY_VEHICLE\x10\x04\x12\x0e\n\nMOTORCYCLE\x10\x05\x12\x0b\n\x07\x42ICYCLE\x10\x06\x12\x0e\n\nPEDESTRIAN\x10\x07\x12\t\n\x05TRAIN\x10\x08\x12\x08\n\x04TAXI\x10\t\"i\n\x05\x42ound\x12!\n\x06points\x18\x01 \x03(\x0b\x32\x11.commonroad.Point\x12=\n\x0cline_marking\x18\x02 \x01(\x0e\x32\'.commonroad.LineMarkingEnum.LineMarking\"\xa3\x01\n\x08StopLine\x12!\n\x06points\x18\x01 \x03(\x0b\x32\x11.commonroad.Point\x12=\n\x0cline_marking\x18\x02 \x02(\x0e\x32\'.commonroad.LineMarkingEnum.LineMarking\x12\x19\n\x11traffic_sign_refs\x18\x03 \x03(\r\x12\x1a\n\x12traffic_light_refs\x18\x04 \x03(\r\"\xe2\x04\n\x07Lanelet\x12\x12\n\nlanelet_id\x18\x01 \x02(\r\x12%\n\nleft_bound\x18\x02 \x02(\x0b\x32\x11.commonroad.Bound\x12&\n\x0bright_bound\x18\x03 \x02(\x0b\x32\x11.commonroad.Bound\x12\x14\n\x0cpredecessors\x18\x04 \x03(\r\x12\x12\n\nsuccessors\x18\x05 \x03(\r\x12\x15\n\radjacent_left\x18\x06 \x01(\r\x12\x16\n\x0e\x61\x64jacent_right\x18\x07 \x01(\r\x12@\n\x11\x61\x64jacent_left_dir\x18\x08 \x01(\x0e\x32%.commonroad.DrivingDirEnum.DrivingDir\x12\x41\n\x12\x61\x64jacent_right_dir\x18\t \x01(\x0e\x32%.commonroad.DrivingDirEnum.DrivingDir\x12\'\n\tstop_line\x18\n \x01(\x0b\x32\x14.commonroad.StopLine\x12>\n\rlanelet_types\x18\x0b \x03(\x0e\x32\'.commonroad.LaneletTypeEnum.LaneletType\x12\x37\n\x0cuser_one_way\x18\x0c \x03(\x0e\x32!.commonroad.RoadUserEnum.RoadUser\x12=\n\x12user_bidirectional\x18\r \x03(\x0e\x32!.commonroad.RoadUserEnum.RoadUser\x12\x19\n\x11traffic_sign_refs\x18\x0e \x03(\r\x12\x1a\n\x12traffic_light_refs\x18\x0f \x03(\r'
  ,
  dependencies=[util__pb2.DESCRIPTOR,])



_LINEMARKINGENUM_LINEMARKING = _descriptor.EnumDescriptor(
  name='LineMarking',
  full_name='commonroad.LineMarkingEnum.LineMarking',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DASHED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SOLID', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BROAD_DASHED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BROAD_SOLID', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_MARKING', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=60,
  serialized_end=160,
)
_sym_db.RegisterEnumDescriptor(_LINEMARKINGENUM_LINEMARKING)

_DRIVINGDIRENUM_DRIVINGDIR = _descriptor.EnumDescriptor(
  name='DrivingDir',
  full_name='commonroad.DrivingDirEnum.DrivingDir',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SAME', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='OPPOSITE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=180,
  serialized_end=216,
)
_sym_db.RegisterEnumDescriptor(_DRIVINGDIRENUM_DRIVINGDIR)

_LANELETTYPEENUM_LANELETTYPE = _descriptor.EnumDescriptor(
  name='LaneletType',
  full_name='commonroad.LaneletTypeEnum.LaneletType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='URBAN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COUNTRY', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HIGHWAY', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DRIVE_WAY', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MAIN_CARRIAGE_WAY', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ACCESS_RAMP', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EXIT_RAMP', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SHOULDER', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS_LANE', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS_STOP', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BICYCLE_LANE', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIDEWALK', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CROSSWALK', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERSTATE', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERSECTION', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BORDER', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARKING', index=16, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESTRICTED', index=17, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=18, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=239,
  serialized_end=536,
)
_sym_db.RegisterEnumDescriptor(_LANELETTYPEENUM_LANELETTYPE)

_ROADUSERENUM_ROADUSER = _descriptor.EnumDescriptor(
  name='RoadUser',
  full_name='commonroad.RoadUserEnum.RoadUser',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='VEHICLE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CAR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRUCK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRIORITY_VEHICLE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MOTORCYCLE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BICYCLE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PEDESTRIAN', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAIN', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TAXI', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=556,
  serialized_end=696,
)
_sym_db.RegisterEnumDescriptor(_ROADUSERENUM_ROADUSER)


_LINEMARKINGENUM = _descriptor.Descriptor(
  name='LineMarkingEnum',
  full_name='commonroad.LineMarkingEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LINEMARKINGENUM_LINEMARKING,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=41,
  serialized_end=160,
)


_DRIVINGDIRENUM = _descriptor.Descriptor(
  name='DrivingDirEnum',
  full_name='commonroad.DrivingDirEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DRIVINGDIRENUM_DRIVINGDIR,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=162,
  serialized_end=216,
)


_LANELETTYPEENUM = _descriptor.Descriptor(
  name='LaneletTypeEnum',
  full_name='commonroad.LaneletTypeEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LANELETTYPEENUM_LANELETTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=219,
  serialized_end=536,
)


_ROADUSERENUM = _descriptor.Descriptor(
  name='RoadUserEnum',
  full_name='commonroad.RoadUserEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _ROADUSERENUM_ROADUSER,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=539,
  serialized_end=696,
)


_BOUND = _descriptor.Descriptor(
  name='Bound',
  full_name='commonroad.Bound',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='commonroad.Bound.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='line_marking', full_name='commonroad.Bound.line_marking', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=698,
  serialized_end=803,
)


_STOPLINE = _descriptor.Descriptor(
  name='StopLine',
  full_name='commonroad.StopLine',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='points', full_name='commonroad.StopLine.points', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='line_marking', full_name='commonroad.StopLine.line_marking', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_sign_refs', full_name='commonroad.StopLine.traffic_sign_refs', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_light_refs', full_name='commonroad.StopLine.traffic_light_refs', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=806,
  serialized_end=969,
)


_LANELET = _descriptor.Descriptor(
  name='Lanelet',
  full_name='commonroad.Lanelet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='lanelet_id', full_name='commonroad.Lanelet.lanelet_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left_bound', full_name='commonroad.Lanelet.left_bound', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right_bound', full_name='commonroad.Lanelet.right_bound', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='predecessors', full_name='commonroad.Lanelet.predecessors', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors', full_name='commonroad.Lanelet.successors', index=4,
      number=5, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_left', full_name='commonroad.Lanelet.adjacent_left', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_right', full_name='commonroad.Lanelet.adjacent_right', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_left_dir', full_name='commonroad.Lanelet.adjacent_left_dir', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='adjacent_right_dir', full_name='commonroad.Lanelet.adjacent_right_dir', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stop_line', full_name='commonroad.Lanelet.stop_line', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lanelet_types', full_name='commonroad.Lanelet.lanelet_types', index=10,
      number=11, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_one_way', full_name='commonroad.Lanelet.user_one_way', index=11,
      number=12, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_bidirectional', full_name='commonroad.Lanelet.user_bidirectional', index=12,
      number=13, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_sign_refs', full_name='commonroad.Lanelet.traffic_sign_refs', index=13,
      number=14, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_light_refs', full_name='commonroad.Lanelet.traffic_light_refs', index=14,
      number=15, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=972,
  serialized_end=1582,
)

_LINEMARKINGENUM_LINEMARKING.containing_type = _LINEMARKINGENUM
_DRIVINGDIRENUM_DRIVINGDIR.containing_type = _DRIVINGDIRENUM
_LANELETTYPEENUM_LANELETTYPE.containing_type = _LANELETTYPEENUM
_ROADUSERENUM_ROADUSER.containing_type = _ROADUSERENUM
_BOUND.fields_by_name['points'].message_type = util__pb2._POINT
_BOUND.fields_by_name['line_marking'].enum_type = _LINEMARKINGENUM_LINEMARKING
_STOPLINE.fields_by_name['points'].message_type = util__pb2._POINT
_STOPLINE.fields_by_name['line_marking'].enum_type = _LINEMARKINGENUM_LINEMARKING
_LANELET.fields_by_name['left_bound'].message_type = _BOUND
_LANELET.fields_by_name['right_bound'].message_type = _BOUND
_LANELET.fields_by_name['adjacent_left_dir'].enum_type = _DRIVINGDIRENUM_DRIVINGDIR
_LANELET.fields_by_name['adjacent_right_dir'].enum_type = _DRIVINGDIRENUM_DRIVINGDIR
_LANELET.fields_by_name['stop_line'].message_type = _STOPLINE
_LANELET.fields_by_name['lanelet_types'].enum_type = _LANELETTYPEENUM_LANELETTYPE
_LANELET.fields_by_name['user_one_way'].enum_type = _ROADUSERENUM_ROADUSER
_LANELET.fields_by_name['user_bidirectional'].enum_type = _ROADUSERENUM_ROADUSER
DESCRIPTOR.message_types_by_name['LineMarkingEnum'] = _LINEMARKINGENUM
DESCRIPTOR.message_types_by_name['DrivingDirEnum'] = _DRIVINGDIRENUM
DESCRIPTOR.message_types_by_name['LaneletTypeEnum'] = _LANELETTYPEENUM
DESCRIPTOR.message_types_by_name['RoadUserEnum'] = _ROADUSERENUM
DESCRIPTOR.message_types_by_name['Bound'] = _BOUND
DESCRIPTOR.message_types_by_name['StopLine'] = _STOPLINE
DESCRIPTOR.message_types_by_name['Lanelet'] = _LANELET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LineMarkingEnum = _reflection.GeneratedProtocolMessageType('LineMarkingEnum', (_message.Message,), {
  'DESCRIPTOR' : _LINEMARKINGENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.LineMarkingEnum)
  })
_sym_db.RegisterMessage(LineMarkingEnum)

DrivingDirEnum = _reflection.GeneratedProtocolMessageType('DrivingDirEnum', (_message.Message,), {
  'DESCRIPTOR' : _DRIVINGDIRENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.DrivingDirEnum)
  })
_sym_db.RegisterMessage(DrivingDirEnum)

LaneletTypeEnum = _reflection.GeneratedProtocolMessageType('LaneletTypeEnum', (_message.Message,), {
  'DESCRIPTOR' : _LANELETTYPEENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.LaneletTypeEnum)
  })
_sym_db.RegisterMessage(LaneletTypeEnum)

RoadUserEnum = _reflection.GeneratedProtocolMessageType('RoadUserEnum', (_message.Message,), {
  'DESCRIPTOR' : _ROADUSERENUM,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.RoadUserEnum)
  })
_sym_db.RegisterMessage(RoadUserEnum)

Bound = _reflection.GeneratedProtocolMessageType('Bound', (_message.Message,), {
  'DESCRIPTOR' : _BOUND,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Bound)
  })
_sym_db.RegisterMessage(Bound)

StopLine = _reflection.GeneratedProtocolMessageType('StopLine', (_message.Message,), {
  'DESCRIPTOR' : _STOPLINE,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.StopLine)
  })
_sym_db.RegisterMessage(StopLine)

Lanelet = _reflection.GeneratedProtocolMessageType('Lanelet', (_message.Message,), {
  'DESCRIPTOR' : _LANELET,
  '__module__' : 'lanelet_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Lanelet)
  })
_sym_db.RegisterMessage(Lanelet)


# @@protoc_insertion_point(module_scope)
