# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: intersection.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='intersection.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12intersection.proto\x12\ncommonroad\"\x9e\x01\n\x08Incoming\x12\x13\n\x0bincoming_id\x18\x01 \x02(\r\x12\x19\n\x11incoming_lanelets\x18\x02 \x03(\r\x12\x18\n\x10successors_right\x18\x03 \x03(\r\x12\x1b\n\x13successors_straight\x18\x04 \x03(\r\x12\x17\n\x0fsuccessors_left\x18\x05 \x03(\r\x12\x12\n\nis_left_of\x18\x06 \x01(\r\"k\n\x0cIntersection\x12\x17\n\x0fintersection_id\x18\x01 \x02(\r\x12\'\n\tincomings\x18\x02 \x03(\x0b\x32\x14.commonroad.Incoming\x12\x19\n\x11\x63rossing_lanelets\x18\x03 \x03(\r'
)




_INCOMING = _descriptor.Descriptor(
  name='Incoming',
  full_name='commonroad.Incoming',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='incoming_id', full_name='commonroad.Incoming.incoming_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='incoming_lanelets', full_name='commonroad.Incoming.incoming_lanelets', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors_right', full_name='commonroad.Incoming.successors_right', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors_straight', full_name='commonroad.Incoming.successors_straight', index=3,
      number=4, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='successors_left', full_name='commonroad.Incoming.successors_left', index=4,
      number=5, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_left_of', full_name='commonroad.Incoming.is_left_of', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=193,
)


_INTERSECTION = _descriptor.Descriptor(
  name='Intersection',
  full_name='commonroad.Intersection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='intersection_id', full_name='commonroad.Intersection.intersection_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='incomings', full_name='commonroad.Intersection.incomings', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='crossing_lanelets', full_name='commonroad.Intersection.crossing_lanelets', index=2,
      number=3, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=195,
  serialized_end=302,
)

_INTERSECTION.fields_by_name['incomings'].message_type = _INCOMING
DESCRIPTOR.message_types_by_name['Incoming'] = _INCOMING
DESCRIPTOR.message_types_by_name['Intersection'] = _INTERSECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Incoming = _reflection.GeneratedProtocolMessageType('Incoming', (_message.Message,), {
  'DESCRIPTOR' : _INCOMING,
  '__module__' : 'intersection_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Incoming)
  })
_sym_db.RegisterMessage(Incoming)

Intersection = _reflection.GeneratedProtocolMessageType('Intersection', (_message.Message,), {
  'DESCRIPTOR' : _INTERSECTION,
  '__module__' : 'intersection_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Intersection)
  })
_sym_db.RegisterMessage(Intersection)


# @@protoc_insertion_point(module_scope)
