syntax = "proto2";

package commonroad;

import "util.proto";

message TrafficSignIDGermanyEnum {
    enum TrafficSignIDGermany {
        WARNING_DANGER_SPOT = 0;
        WARNING_RIGHT_BEFORE_LEFT = 1;
        WARNING_LEFT_CURVE = 2;
        WARNING_RIGHT_CURVE = 3;
        WARNING_STEEP_HILL_DOWNWARDS = 4;
        WARNING_SLIPPERY_ROAD = 5;
        WARNING_CONSTRUCTION_SITE = 6;
        WARNING_TRAFFIC_QUEUES_LIKELY = 7;
        WARNING_ONCOMING_TRAFFIC = 8;
        WARNING_TRAFFIC_LIGHTS_AHEAD = 9;
        WARNING_PEDESTRIANS_RIGHT = 10;
        WARNING_PEDESTRIANS_LEFT = 11;
        WARNING_CROSSING_CYCLIST = 12;
        WARNING_ANIMAL_CROSSING_RIGHT = 13;
        WARNING_LOOSE_GRAVEL = 14;
        RAILWAY = 15;
        YIELD = 16;
        STOP = 17;
        PRIORITY_OPPOSITE_DIRECTION = 18;
        TURN_RIGHT_AHEAD = 19;
        TURN_LEFT_AHEAD = 20;
        KEEP_STRAIGHT_AHEAD = 21;
        PRESCRIBED_DIRECTION_RIGHT = 22;
        ROUNDABOUT = 23;
        ONEWAY_RIGHT = 24;
        ONEWAY_LEFT = 25;
        PRESCRIBED_PASSING_LEFT = 26;
        PRESCRIBED_PASSING_RIGHT = 27;
        DO_NOT_DRIVE_ON_SHOULDER_LANE = 28;
        DO_NOT_DRIVE_ON_SHOULDER_LANE_2_LANE = 29;
        DO_NOT_DRIVE_ON_SHOULDER_LANE_3_LANE = 30;
        BUS_STOP = 31;
        BIKEWAY = 32;
        PEDESTRIAN_SIDEWALK = 33;
        PEDESTRIAN_AND_BICYCLE_ROAD = 34;
        PEDESTRIAN_ZONE_START = 35;
        PEDESTRIAN_ZONE_END = 36;
        BICYCLE_ROAD_START = 37;
        BICYCLE_ROAD_END = 38;
        BUS_LANE = 39;
        BAN_ALL_VEHICLES = 40;
        BAN_CARS = 41;
        BAN_TRUCKS = 42;
        BAN_BICYCLE = 43;
        BAN_MOTORCYCLE = 44;
        BAN_BUS = 45;
        BAN_PEDESTRIAN = 46;
        BAN_CAR_TRUCK_BUS_MOTORCYCLE = 47;
        BAN_VEHICLES_CARRYING_DANGEROUS_GOODS = 48;
        MAX_WEIGHT = 49;
        MAX_WIDTH = 50;
        MAX_HEIGHT = 51;
        MAX_LENGTH = 52;
        NO_ENTRY = 53;
        ENVIRONMENTAL_ZONE_START = 54;
        ENVIRONMENTAL_ZONE_END = 55;
        U_TURN = 56;
        MAX_SPEED = 57;
        MAX_SPEED_ZONE_START = 58;
        MAX_SPEED_ZONE_END = 59;
        MIN_SPEED = 60;
        NO_OVERTAKING_START = 61;
        NO_OVERTAKING_TRUCKS_START = 62;
        MAX_SPEED_END = 63;
        NO_OVERTAKING_END = 64;
        NO_OVERTAKING_TRUCKS_END = 65;
        ALL_MAX_SPEED_AND_OVERTAKING_END = 66;
        NO_STOP_START_RIGHT = 67;
        NO_STOP_CENTER_RIGHT = 68;
        RESTRICTED_STOP_CENTER_RIGHT = 69;
        RIGHT_OF_WAY = 70;
        PRIORITY = 71;
        PRIORITY_OVER_ONCOMING = 72;
        TOWN_SIGN = 73;
        TOWN_SIGN_BACK = 74;
        PARKING_AREA = 75;
        PARKING_AREA_LEFT = 76;
        PARKING_AREA_RIGHT = 77;
        PARKING_AREA_RIGHT_LEFT = 78;
        TRAFFIC_CALMED_AREA_START = 79;
        TRAFFIC_CALMED_AREA_END = 80;
        TUNNEL = 81;
        EMERYGECNY_STOP = 82;
        INTERSTATE_START = 83;
        INTERSTATE_END = 84;
        HIGHWAY_START = 85;
        HIGHWAY_END = 86;
        HIGHWAY_EXIT_WITH_PLACE_NAME = 87;
        EXIT_ROUTE = 88;
        HIGHWAY_EXIT = 89;
        EXIT_BUILT_UP = 90;
        EXIT_GENERAL = 91;
        PEDESTRIANS_CROSSING = 92;
        WATER_PROTECTION_ZONE = 93;
        TRAFFIC_ASSISTANTS = 94;
        DEAD_END = 95;
        POLICE = 96;
        EMERGENCY_CALL_STATION = 97;
        GAS_STATION = 98;
        CAMP_AND_CARAVAN_SITE = 99;
        ATTRACTION_POINT = 100;
        TOURISTIC_ROUTE = 101;
        NEARBY_ATTRACTION_POINT = 102;
        HIGHWAY_INTERSECTION = 103;
        DIRECTION_ARROW_SIGN_MULTI = 104;
        DIRECTION_ARROW_SIGN_SINGLE = 105;
        DIRECTION_SIGN_CONSOLIDATED = 106;
        EXPRESSWAY_ARROW_DIRECTION = 107;
        ARROW_SIGN_POST_POINT_OF_INTEREST_LEFT = 108;
        STATION = 109;
        GUIDE_SIGN_TABLE = 110;
        ADVANCE_DIRECTION = 111;
        DIRECTIONS_SIGN = 112;
        EXPRESSWAY_ENTRANCE_DIRECTIONS = 113;
        INTERSTATE_ANNOUNCEMENT = 114;
        INTERSTATE_ADVANCE_DIRECTION = 115;
        HIGHWAY_EXIT_AHEAD_100_METER = 116;
        HIGHWAY_EXIT_AHEAD_200_METER = 117;
        HIGHWAY_EXIT_AHEAD_300_METER = 118;
        EXPRESSWAY_EXIT_100_METRES = 119;
        EXPRESSWAY_EXIT_200_METRES = 120;
        EXPRESSWAY_EXIT_300_METRES = 121;
        INTERSTATE_DISTANCE = 122;
        DETOUR_SKETCH = 123;
        DETOUR_STRAIGHT = 124;
        DETOUR_ON_DEMAND_LEFT = 125;
        DETOUR_ON_DEMAND_GET_IN_LEFT_LANE = 126;
        DETOUR_ON_DEMAND_ANNOUNCEMENT_RIGHT = 127;
        DETOUR_ON_DEMAND_RIGHT = 128;
        DETOUR_ON_DEMAND_GET_IN_RIGHT_LANE = 129;
        DETOUR_ON_DEMAND_STRAIGHTFORWARD = 130;
        TRANSITION_3_LEFT_2_TRANSITIONED = 131;
        TRANSITION_1_LEFT_1_STRAIGHT = 132;
        TRANSITION_3_RIGHT = 133;
        LANE_BOARD_NO_OPPOSITE_TWO_LANES = 134;
        THREE_LANES_NO_ONCOMING_LANES = 135;
        FOUR_LANES_NO_ONCOMING_LANES = 136;
        FIVE_LANES_NO_ONCOMING_LANES = 137;
        LANE_BOARD_3_LANES_NO_OPPOSITE_WITH_SIGNS = 138;
        NARROWING_LANES_1_LANE_FROM_RIGHT = 139;
        NARROWING_LANES_1_LANE_FROM_LEFT = 140;
        NARROWING_LANES_2_LANES_PLUS_1_LEFT = 141;
        FOUR_LANES_NO_ONCOMING_TRAFFIC_TWO_RIGHT_LANES_TURN_RIGHT = 142;
        MERGING_LANES_1_LANE_PLUS_1_LANE_RIGHT = 143;
        BARRIER = 144;
        BARRIER_GATE_100_800 = 145;
        BARRIER_GATE_100_1200 = 146;
        BARRIER_GATE_100_1600 = 147;
        BARRIER_GATE_250_1600 = 148;
        BARRIER_GATE = 149;
        ROAD_WARNING_POST_SCRAPER_BEACON_RIGHT = 150;
        ROAD_WARNING_POST_ARROW_BEACON_RIGHT = 151;
        ROAD_WARNING_POST_SCRAPER_BEACON_LEFT = 152;
        ROAD_WARNING_POST_SCRAPER_BEACON_ARROW_RIGHT = 153;
        ROAD_WARNING_POST_GUIDE_UP_THREE_ARROWS = 154;
        DIRECTION_SIGN_LEFT_SINGLE = 155;
        DIRECTION_SIGN_LEFT_SMALL = 156;
        DIRECTION_SIGN_LEFT_MEDIUM = 157;
        DIRECTION_SIGN_LEFT_LARGE = 158;
        DIRECTION_SIGN_RIGHT_SINGLE = 159;
        DIRECTION_SIGN_RIGHT_SMALL = 160;
        DIRECTION_SIGN_RIGHT_MEDIUM = 161;
        DIRECTION_SIGN_RIGHT_LARGE = 162;
        WARNING_PANEL_RIGHT = 163;
        WARNING_PANEL_LEFT = 164;
        WARNING_PANEL_STRAIGHT_BROAD = 165;
        WARNING_PANEL_STRAIGHT_HIGH = 166;
        GUIDE_SILL_WITH_GUIDE_BEACON_RIGHT = 167;
        GUIDE_RAIL_WITH_GUIDE_BEACON_RIGHT = 168;
        GUIDE_PANEL_WITH_GUIDE_BEACON_RIGHT = 169;
        GREEN_ARROW = 170;
        ADDITION_LEFT_DIRECTION = 171;
        ADDITION_LEFT_DIRECTION_1 = 172;
        ADDITION_LEFT_DIRECTION_DANGER_POINT = 173;
        ADDITION_RIGHT_DIRECTION_1 = 174;
        ADDITION_RIGHT_DIRECTION_DANGER_POINT = 175;
        ADDITION_BOTH_DIRECTIONS_HORIZONTAL = 176;
        ADDITION_BOTH_DIRECTIONS_VERTICAL = 177;
        ADDITION_VALID_FOR_X_METERS = 178;
        ADDITION_VALID_FOR_X_KILOMETERS = 179;
        ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_RIGHT_YIELD = 180;
        ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_RIGHT_YIELD = 181;
        ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = 182;
        ADDITION_LEFT_TURNING_PRIORITY_WITH_RIGHT_YIELD = 183;
        ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = 184;
        ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_LEFT_YIELD = 185;
        ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_LEFT_YIELD = 186;
        ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = 187;
        ADDITION_RIGHT_TURNING_PRIORITY_WITH_LEFT_YIELD = 188;
        ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = 189;
        ADDITION_VALID_IN_X_METERS = 190;
        ADDITION_VALID_IN_X_KILOMETERS = 191;
        ADDITION_VALID_IN_200_KILOMETERS = 192;
        ADDITION_VALID_IN_400_METRES = 193;
        ADDITION_VALID_IN_600_METRES = 194;
        ADDITION_VALID_IN_2_KILOMETERS = 195;
        ADDITION_OIL_ON_ROAD = 196;
        ADDITION_SMOKE = 197;
        ADDITION_LOOSE_GRAVEL = 198;
        ADDITION_BUILDING_SITE_EXIT = 199;
        ADDITION_DAMAGED_ROAD = 201;
        ADDITION_DIRTY_ROAD = 202;
        ADDITION_DANGER_OF_COLLISION = 203;
        ADDITION_TOAD_MIGRATION = 204;
        ADDITION_DANGER_OF_CONGESTION = 205;
        ADDITION_RESTRICTED_VIEW_DUE_TO_TREES = 206;
        DANGER_INDICATION_SMOKE = 207;
        ADDITION_CHILDREN_PLAYING_ON_ROAD = 208;
        ADDITION_WINTER_SPORTS_ALLOWED = 209;
        ADDITION_TRAILERS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = 210;
        ADDITION_CARAVANS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = 211;
        ADDITION_ROLLING_HIGHWAY = 212;
        ADDITION_LOADING_AREA = 213;
        ADDITION_END = 214;
        ADDITION_GET_OFF_BICYCLES = 215;
        ADDITION_NO_MOPEDS = 216;
        ADDITION_GREEN_WAVE_AT_KM_H = 217;
        ADDITION_STOP_HERE_AT_RED = 218;
        ADDITION_NOISE_CONTROL = 219;
        ADDITION_INFLOW_REGULATION = 220;
        ADDITION_SECONDARY_LANE = 221;
        ADDITION_SCHOOL = 222;
        ADDITION_KINDERGARTEN = 223;
        ADDITION_RETIREMENT_HOME = 224;
        ADDITION_HOSPITAL = 225;
        ADDITION_RESIDENTS_PERMITTED = 226;
        ADDITION_BICYCLES_PERMITTED = 227;
        ADDITION_CARS_PERMITTED = 228;
        ADDITION_AGRICULTURE_PERMITTED = 229;
        ADDITION_FOREST_PERMITTED = 230;
        ADDITION_AGRICULTURE_FOREST_PERMITTED = 231;
        ADDITION_GREEN_STICKER_PERMITTED = 232;
        ADDITION_TIME_PERIOD_PERMITTED = 233;
        ADDITION_MOTOR_VEHICLES_ALLOWED_MASS_3_5_TONS = 234;
        ADDITION_MIN_MASS_3_5_TONS = 235;
        ADDITION_NO_WATER_POLLUTANTS_LOADED = 236;
        ALLOWED_MASS_7_5_TONS = 237;
        ADDITION_VALID_ON_SHOULDER = 238;
        ADDITION_VALID_WHEN_WET = 239;
        LINE_MARKING_MISSING = 240;
        UNKNOWN = 241;
    }
}

message TrafficSignIDZamundaEnum {
    enum TrafficSignIDZamunda {
        WARNING_DANGER_SPOT = 0;
        WARNING_RIGHT_BEFORE_LEFT = 1;
        WARNING_LEFT_CURVE = 2;
        WARNING_RIGHT_CURVE = 3;
        WARNING_STEEP_HILL_DOWNWARDS = 4;
        WARNING_SLIPPERY_ROAD = 5;
        WARNING_CONSTRUCTION_SITE = 6;
        WARNING_TRAFFIC_QUEUES_LIKELY = 7;
        WARNING_ONCOMING_TRAFFIC = 8;
        WARNING_TRAFFIC_LIGHTS_AHEAD = 9;
        WARNING_PEDESTRIANS_RIGHT = 10;
        WARNING_PEDESTRIANS_LEFT = 11;
        WARNING_CROSSING_CYCLIST = 12;
        WARNING_ANIMAL_CROSSING_RIGHT = 13;
        WARNING_LOOSE_GRAVEL = 14;
        RAILWAY = 15;
        YIELD = 16;
        STOP = 17;
        PRIORITY_OPPOSITE_DIRECTION = 18;
        TURN_RIGHT_AHEAD = 19;
        TURN_LEFT_AHEAD = 20;
        KEEP_STRAIGHT_AHEAD = 21;
        PRESCRIBED_DIRECTION_RIGHT = 22;
        ROUNDABOUT = 23;
        ONEWAY_RIGHT = 24;
        ONEWAY_LEFT = 25;
        PRESCRIBED_PASSING_LEFT = 26;
        PRESCRIBED_PASSING_RIGHT = 27;
        DO_NOT_DRIVE_ON_SHOULDER_LANE = 28;
        DO_NOT_DRIVE_ON_SHOULDER_LANE_2_LANE = 29;
        DO_NOT_DRIVE_ON_SHOULDER_LANE_3_LANE = 30;
        BUS_STOP = 31;
        BIKEWAY = 32;
        PEDESTRIAN_SIDEWALK = 33;
        PEDESTRIAN_AND_BICYCLE_ROAD = 34;
        PEDESTRIAN_ZONE_START = 35;
        PEDESTRIAN_ZONE_END = 36;
        BICYCLE_ROAD_START = 37;
        BICYCLE_ROAD_END = 38;
        BUS_LANE = 39;
        BAN_ALL_VEHICLES = 40;
        BAN_CARS = 41;
        BAN_TRUCKS = 42;
        BAN_BICYCLE = 43;
        BAN_MOTORCYCLE = 44;
        BAN_BUS = 45;
        BAN_PEDESTRIAN = 46;
        BAN_CAR_TRUCK_BUS_MOTORCYCLE = 47;
        BAN_VEHICLES_CARRYING_DANGEROUS_GOODS = 48;
        MAX_WEIGHT = 49;
        MAX_WIDTH = 50;
        MAX_HEIGHT = 51;
        MAX_LENGTH = 52;
        NO_ENTRY = 53;
        ENVIRONMENTAL_ZONE_START = 54;
        ENVIRONMENTAL_ZONE_END = 55;
        U_TURN = 56;
        MAX_SPEED = 57;
        MAX_SPEED_ZONE_START = 58;
        MAX_SPEED_ZONE_END = 59;
        MIN_SPEED = 60;
        NO_OVERTAKING_START = 61;
        NO_OVERTAKING_TRUCKS_START = 62;
        MAX_SPEED_END = 63;
        NO_OVERTAKING_END = 64;
        NO_OVERTAKING_TRUCKS_END = 65;
        ALL_MAX_SPEED_AND_OVERTAKING_END = 66;
        NO_STOP_START_RIGHT = 67;
        NO_STOP_CENTER_RIGHT = 68;
        RESTRICTED_STOP_CENTER_RIGHT = 69;
        RIGHT_OF_WAY = 70;
        PRIORITY = 71;
        PRIORITY_OVER_ONCOMING = 72;
        TOWN_SIGN = 73;
        TOWN_SIGN_BACK = 74;
        PARKING_AREA = 75;
        PARKING_AREA_LEFT = 76;
        PARKING_AREA_RIGHT = 77;
        PARKING_AREA_RIGHT_LEFT = 78;
        TRAFFIC_CALMED_AREA_START = 79;
        TRAFFIC_CALMED_AREA_END = 80;
        TUNNEL = 81;
        EMERGENCY_STOP = 82;
        INTERSTATE_START = 83;
        INTERSTATE_END = 84;
        HIGHWAY_START = 85;
        HIGHWAY_END = 86;
        HIGHWAY_EXIT_WITH_PLACE_NAME = 87;
        EXIT_ROUTE = 88;
        HIGHWAY_EXIT = 89;
        EXIT_BUILT_UP = 90;
        EXIT_GENERAL = 91;
        PEDESTRIANS_CROSSING = 92;
        WATER_PROTECTION_ZONE = 93;
        TRAFFIC_ASSISTANTS = 94;
        DEAD_END = 95;
        POLICE = 96;
        EMERGENCY_CALL_STATION = 97;
        GAS_STATION = 98;
        CAMP_AND_CARAVAN_SITE = 99;
        ATTRACTION_POINT = 100;
        TOURISTIC_ROUTE = 101;
        NEARBY_ATTRACTION_POINT = 102;
        HIGHWAY_INTERSECTION = 103;
        DIRECTION_ARROW_SIGN_MULTI = 104;
        DIRECTION_ARROW_SIGN_SINGLE = 105;
        DIRECTION_SIGN_CONSOLIDATED = 106;
        EXPRESSWAY_ARROW_DIRECTION = 107;
        ARROW_SIGN_POST_POINT_OF_INTEREST_LEFT = 108;
        STATION = 109;
        GUIDE_SIGN_TABLE = 110;
        ADVANCE_DIRECTION = 111;
        DIRECTIONS_SIGN = 112;
        EXPRESSWAY_ENTRANCE_DIRECTIONS = 113;
        INTERSTATE_ANNOUNCEMENT = 114;
        INTERSTATE_ADVANCE_DIRECTION = 115;
        HIGHWAY_EXIT_AHEAD_100_METER = 116;
        HIGHWAY_EXIT_AHEAD_200_METER = 117;
        HIGHWAY_EXIT_AHEAD_300_METER = 118;
        EXPRESSWAY_EXIT_100_METRES = 119;
        EXPRESSWAY_EXIT_200_METRES = 120;
        EXPRESSWAY_EXIT_300_METRES = 121;
        INTERSTATE_DISTANCE = 122;
        DETOUR_SKETCH = 123;
        DETOUR_STRAIGHT = 124;
        DETOUR_ON_DEMAND_LEFT = 125;
        DETOUR_ON_DEMAND_GET_IN_LEFT_LANE = 126;
        DETOUR_ON_DEMAND_ANNOUNCEMENT_RIGHT = 127;
        DETOUR_ON_DEMAND_RIGHT = 128;
        DETOUR_ON_DEMAND_GET_IN_RIGHT_LANE = 129;
        DETOUR_ON_DEMAND_STRAIGHTFORWARD = 130;
        TRANSITION_3_LEFT_2_TRANSITIONED = 131;
        TRANSITION_1_LEFT_1_STRAIGHT = 132;
        TRANSITION_3_RIGHT = 133;
        LANE_BOARD_NO_OPPOSITE_TWO_LANES = 134;
        THREE_LANES_NO_ONCOMING_LANES = 135;
        FOUR_LANES_NO_ONCOMING_LANES = 136;
        FIVE_LANES_NO_ONCOMING_LANES = 137;
        LANE_BOARD_3_LANES_NO_OPPOSITE_WITH_SIGNS = 138;
        NARROWING_LANES_1_LANE_FROM_RIGHT = 139;
        NARROWING_LANES_1_LANE_FROM_LEFT = 140;
        NARROWING_LANES_2_LANES_PLUS_1_LEFT = 141;
        FOUR_LANES_NO_ONCOMING_TRAFFIC_TWO_RIGHT_LANES_TURN_RIGHT = 142;
        MERGING_LANES_1_LANE_PLUS_1_LANE_RIGHT = 143;
        BARRIER = 144;
        BARRIER_GATE_100_800 = 145;
        BARRIER_GATE_100_1200 = 146;
        BARRIER_GATE_100_1600 = 147;
        BARRIER_GATE_250_1600 = 148;
        BARRIER_GATE = 149;
        ROAD_WARNING_POST_SCRAPER_BEACON_RIGHT = 150;
        ROAD_WARNING_POST_ARROW_BEACON_RIGHT = 151;
        ROAD_WARNING_POST_SCRAPER_BEACON_LEFT = 152;
        ROAD_WARNING_POST_SCRAPER_BEACON_ARROW_RIGHT = 153;
        ROAD_WARNING_POST_GUIDE_UP_THREE_ARROWS = 154;
        DIRECTION_SIGN_LEFT_SINGLE = 155;
        DIRECTION_SIGN_LEFT_SMALL = 156;
        DIRECTION_SIGN_LEFT_MEDIUM = 157;
        DIRECTION_SIGN_LEFT_LARGE = 158;
        DIRECTION_SIGN_RIGHT_SINGLE = 159;
        DIRECTION_SIGN_RIGHT_SMALL = 160;
        DIRECTION_SIGN_RIGHT_MEDIUM = 161;
        DIRECTION_SIGN_RIGHT_LARGE = 162;
        WARNING_PANEL_RIGHT = 163;
        WARNING_PANEL_LEFT = 164;
        WARNING_PANEL_STRAIGHT_BROAD = 165;
        WARNING_PANEL_STRAIGHT_HIGH = 166;
        GUIDE_SILL_WITH_GUIDE_BEACON_RIGHT = 167;
        GUIDE_RAIL_WITH_GUIDE_BEACON_RIGHT = 168;
        GUIDE_PANEL_WITH_GUIDE_BEACON_RIGHT = 169;
        GREEN_ARROW = 170;
        ADDITION_LEFT_DIRECTION = 171;
        ADDITION_LEFT_DIRECTION_1 = 172;
        ADDITION_LEFT_DIRECTION_DANGER_POINT = 173;
        ADDITION_RIGHT_DIRECTION_1 = 174;
        ADDITION_RIGHT_DIRECTION_DANGER_POINT = 175;
        ADDITION_BOTH_DIRECTIONS_HORIZONTAL = 176;
        ADDITION_BOTH_DIRECTIONS_VERTICAL = 177;
        ADDITION_VALID_FOR_X_METERS = 178;
        ADDITION_VALID_FOR_X_KILOMETERS = 179;
        ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_RIGHT_YIELD = 180;
        ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_RIGHT_YIELD = 181;
        ADDITION_LEFT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = 182;
        ADDITION_LEFT_TURNING_PRIORITY_WITH_RIGHT_YIELD = 183;
        ADDITION_LEFT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = 184;
        ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_LEFT_YIELD = 185;
        ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_LEFT_YIELD = 186;
        ADDITION_RIGHT_TURNING_PRIORITY_WITH_OPPOSITE_YIELD = 187;
        ADDITION_RIGHT_TURNING_PRIORITY_WITH_LEFT_YIELD = 188;
        ADDITION_RIGHT_TRAFFIC_PRIORITY_WITH_STRAIGHT_YIELD = 189;
        ADDITION_VALID_IN_X_METERS = 190;
        ADDITION_VALID_IN_X_KILOMETERS = 191;
        ADDITION_VALID_IN_200_KILOMETERS = 192;
        ADDITION_VALID_IN_400_METRES = 193;
        ADDITION_VALID_IN_600_METRES = 194;
        ADDITION_VALID_IN_2_KILOMETERS = 195;
        ADDITION_OIL_ON_ROAD = 196;
        ADDITION_SMOKE = 197;
        ADDITION_LOOSE_GRAVEL = 198;
        ADDITION_BUILDING_SITE_EXIT = 199;
        ADDITION_DAMAGED_ROAD = 201;
        ADDITION_DIRTY_ROAD = 202;
        ADDITION_DANGER_OF_COLLISION = 203;
        ADDITION_TOAD_MIGRATION = 204;
        ADDITION_DANGER_OF_CONGESTION = 205;
        ADDITION_RESTRICTED_VIEW_DUE_TO_TREES = 206;
        DANGER_INDICATION_SMOKE = 207;
        ADDITION_CHILDREN_PLAYING_ON_ROAD = 208;
        ADDITION_WINTER_SPORTS_ALLOWED = 209;
        ADDITION_TRAILERS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = 210;
        ADDITION_CARAVANS_ALLOWED_TO_PARK_MORE_THAN_14_DAYS = 211;
        ADDITION_ROLLING_HIGHWAY = 212;
        ADDITION_LOADING_AREA = 213;
        ADDITION_END = 214;
        ADDITION_GET_OFF_BICYCLES = 215;
        ADDITION_NO_MOPEDS = 216;
        ADDITION_GREEN_WAVE_AT_KM_H = 217;
        ADDITION_STOP_HERE_AT_RED = 218;
        ADDITION_NOISE_CONTROL = 219;
        ADDITION_INFLOW_REGULATION = 220;
        ADDITION_SECONDARY_LANE = 221;
        ADDITION_SCHOOL = 222;
        ADDITION_KINDERGARTEN = 223;
        ADDITION_RETIREMENT_HOME = 224;
        ADDITION_HOSPITAL = 225;
        ADDITION_RESIDENTS_PERMITTED = 226;
        ADDITION_BICYCLES_PERMITTED = 227;
        ADDITION_CARS_PERMITTED = 228;
        ADDITION_AGRICULTURE_PERMITTED = 229;
        ADDITION_FOREST_PERMITTED = 230;
        ADDITION_AGRICULTURE_FOREST_PERMITTED = 231;
        ADDITION_GREEN_STICKER_PERMITTED = 232;
        ADDITION_TIME_PERIOD_PERMITTED = 233;
        ADDITION_MOTOR_VEHICLES_ALLOWED_MASS_3_5_TONS = 234;
        ADDITION_MIN_MASS_3_5_TONS = 235;
        ADDITION_NO_WATER_POLLUTANTS_LOADED = 236;
        ALLOWED_MASS_7_5_TONS = 237;
        ADDITION_VALID_ON_SHOULDER = 238;
        ADDITION_VALID_WHEN_WET = 239;
        LINE_MARKING_MISSING = 240;
        UNKNOWN = 241;
    }
}

message TrafficSignIDUsaEnum {
    enum TrafficSignIDUsa {
        MAX_SPEED = 0;
        U_TURN = 1;
        ROAD_WORK_AHEAD = 2;
        NO_PARKING_ANY_TIME = 3;
        NO_STANDING = 4;
        TOW_AWAY_ZONE = 5;
        ONE_WAY_LEFT = 6;
        ONE_WAY_RIGHT = 7;
        DO_NOT_ENTER = 8;
        NO_LEFT_TURN = 9;
        RIGHT_TURN_ONLY = 10;
        TURN_ONLY_LANES = 11;
        NO_RIGHT_TURN = 12;
        KEEP_RIGHT = 13;
        SIGNAL_AHEAD = 14;
        LOADING_ZONE = 15;
        NO_PARKING = 16;
        LEFT_TURN_ONLY = 17;
        NO_STRAIGHT_THROUGH = 18;
        LEFT_REVERSE_TURN = 19;
        PEDESTRIAN_WARNING = 20;
        DIAGONAL_DOWNWARD_LEFT_ARROW = 21;
        RIGHT_LANE_ENDS = 22;
        RESERVED_HANDICAP_PARKING = 23;
        NO_PARKING_BUS_STOP = 24;
        ON_PAVEMENT = 25;
        DO_NOT_BLOCK_INTERSECTION = 26;
        WARNING_ARROW_LEFT = 27;
        UNKNOWN = 28;
    }
}

message TrafficSignIDChinaEnum {
    enum TrafficSignIDChina {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDSpainEnum {
    enum TrafficSignIDSpain {
        YIELD = 0;
        STOP = 1;
        BAN_ALL_VEHICLES = 2;
        NO_ENTRY = 3;
        BAN_TRUCKS = 4;
        MAX_WEIGHT = 5;
        MAX_HEIGHT = 6;
        MAX_SPEED = 7;
        NO_OVERTAKING_START = 8;
        NO_STOPPING = 9;
        NO_WAITING = 10;
        PEDESTRIANS_CROSSING = 11;
        UNKNOWN = 12;
    }
}

message TrafficSignIDRussiaEnum {
    enum TrafficSignIDRussia {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDArgentinaEnum {
    enum TrafficSignIDArgentina {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDBelgiumEnum {
    enum TrafficSignIDBelgium {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDFranceEnum {
    enum TrafficSignIDFrance {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDGreeceEnum {
    enum TrafficSignIDGreece {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDCroatiaEnum {
    enum TrafficSignIDCroatia {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDItalyEnum {
    enum TrafficSignIDItaly {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignIDPuertoRicoEnum {
    enum TrafficSignIDPuertoRico {
        MAX_SPEED = 0;
        UNKNOWN = 1;
    }
}

message TrafficSignElement {
    oneof traffic_sign_element_id {
        TrafficSignIDGermanyEnum.TrafficSignIDGermany germany_element_id = 1;
        TrafficSignIDZamundaEnum.TrafficSignIDZamunda zamunda_element_id = 2;
        TrafficSignIDUsaEnum.TrafficSignIDUsa usa_element_id = 3;
        TrafficSignIDChinaEnum.TrafficSignIDChina china_element_id = 4;
        TrafficSignIDSpainEnum.TrafficSignIDSpain spain_element_id = 5;
        TrafficSignIDRussiaEnum.TrafficSignIDRussia russia_element_id = 7;
        TrafficSignIDArgentinaEnum.TrafficSignIDArgentina argentina_element_id = 8;
        TrafficSignIDBelgiumEnum.TrafficSignIDBelgium belgium_element_id = 9;
        TrafficSignIDFranceEnum.TrafficSignIDFrance france_element_id = 10;
        TrafficSignIDGreeceEnum.TrafficSignIDGreece greece_element_id = 11;
        TrafficSignIDCroatiaEnum.TrafficSignIDCroatia croatia_element_id = 12;
        TrafficSignIDItalyEnum.TrafficSignIDItaly italy_element_id = 13;
        TrafficSignIDPuertoRicoEnum.TrafficSignIDPuertoRico puerto_rico_element_id = 14;
    }
    repeated string additional_values = 15;
}

message TrafficSign {
    required uint32  traffic_sign_id = 1;
    repeated TrafficSignElement traffic_sign_elements = 2;
    repeated uint32 first_occurrences = 3;
    optional Point position = 4;
    optional bool virtual = 5;
}