# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scenario_tags.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='scenario_tags.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13scenario_tags.proto\x12\ncommonroad\"\xdd\x03\n\x07TagEnum\"\xd1\x03\n\x03Tag\x12\x0e\n\nINTERSTATE\x10\x00\x12\t\n\x05URBAN\x10\x01\x12\x0b\n\x07HIGHWAY\x10\x02\x12\x0b\n\x07\x43OMFORT\x10\x03\x12\x0c\n\x08\x43RITICAL\x10\x04\x12\x0b\n\x07\x45VASIVE\x10\x05\x12\n\n\x06\x43UT_IN\x10\x06\x12\x11\n\rILLEGAL_CUTIN\x10\x07\x12\x10\n\x0cINTERSECTION\x10\x08\x12\x0f\n\x0bLANE_CHANGE\x10\t\x12\x12\n\x0eLANE_FOLLOWING\x10\n\x12\x11\n\rMERGING_LANES\x10\x0b\x12\x0e\n\nMULTI_LANE\x10\x0c\x12\x14\n\x10ONCOMING_TRAFFIC\x10\r\x12\x17\n\x13NO_ONCOMING_TRAFFIC\x10\x0e\x12\x12\n\x0ePARALLEL_LANES\x10\x0f\x12\x0e\n\nRACE_TRACK\x10\x10\x12\x0e\n\nROUNDABOUT\x10\x11\x12\t\n\x05RURAL\x10\x12\x12\r\n\tSIMULATED\x10\x13\x12\x0f\n\x0bSINGLE_LANE\x10\x14\x12\r\n\tSLIP_ROAD\x10\x15\x12\x0f\n\x0bSPEED_LIMIT\x10\x16\x12\x0f\n\x0bTRAFFIC_JAM\x10\x17\x12\r\n\tTURN_LEFT\x10\x18\x12\x0e\n\nTURN_RIGHT\x10\x19\x12\x0c\n\x08TWO_LANE\x10\x1a\x12\x15\n\x11\x45MERGENCY_BRAKING\x10\x1b\"5\n\x0cScenarioTags\x12%\n\x04tags\x18\x01 \x03(\x0e\x32\x17.commonroad.TagEnum.Tag'
)



_TAGENUM_TAG = _descriptor.EnumDescriptor(
  name='Tag',
  full_name='commonroad.TagEnum.Tag',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='INTERSTATE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='URBAN', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='HIGHWAY', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='COMFORT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CRITICAL', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVASIVE', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CUT_IN', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ILLEGAL_CUTIN', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INTERSECTION', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LANE_CHANGE', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LANE_FOLLOWING', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MERGING_LANES', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MULTI_LANE', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ONCOMING_TRAFFIC', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='NO_ONCOMING_TRAFFIC', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARALLEL_LANES', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RACE_TRACK', index=16, number=16,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROUNDABOUT', index=17, number=17,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RURAL', index=18, number=18,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SIMULATED', index=19, number=19,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SINGLE_LANE', index=20, number=20,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SLIP_ROAD', index=21, number=21,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SPEED_LIMIT', index=22, number=22,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAFFIC_JAM', index=23, number=23,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TURN_LEFT', index=24, number=24,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TURN_RIGHT', index=25, number=25,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TWO_LANE', index=26, number=26,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EMERGENCY_BRAKING', index=27, number=27,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=48,
  serialized_end=513,
)
_sym_db.RegisterEnumDescriptor(_TAGENUM_TAG)


_TAGENUM = _descriptor.Descriptor(
  name='TagEnum',
  full_name='commonroad.TagEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TAGENUM_TAG,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=513,
)


_SCENARIOTAGS = _descriptor.Descriptor(
  name='ScenarioTags',
  full_name='commonroad.ScenarioTags',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tags', full_name='commonroad.ScenarioTags.tags', index=0,
      number=1, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=515,
  serialized_end=568,
)

_TAGENUM_TAG.containing_type = _TAGENUM
_SCENARIOTAGS.fields_by_name['tags'].enum_type = _TAGENUM_TAG
DESCRIPTOR.message_types_by_name['TagEnum'] = _TAGENUM
DESCRIPTOR.message_types_by_name['ScenarioTags'] = _SCENARIOTAGS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TagEnum = _reflection.GeneratedProtocolMessageType('TagEnum', (_message.Message,), {
  'DESCRIPTOR' : _TAGENUM,
  '__module__' : 'scenario_tags_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TagEnum)
  })
_sym_db.RegisterMessage(TagEnum)

ScenarioTags = _reflection.GeneratedProtocolMessageType('ScenarioTags', (_message.Message,), {
  'DESCRIPTOR' : _SCENARIOTAGS,
  '__module__' : 'scenario_tags_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.ScenarioTags)
  })
_sym_db.RegisterMessage(ScenarioTags)


# @@protoc_insertion_point(module_scope)
