# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: planning_problem.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.obstacle_pb2 as obstacle__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='planning_problem.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x16planning_problem.proto\x12\ncommonroad\x1a\x0eobstacle.proto\"M\n\tGoalState\x12 \n\x05state\x18\x01 \x02(\x0b\x32\x11.commonroad.State\x12\x1e\n\x16goal_position_lanelets\x18\x02 \x03(\r\"\x84\x01\n\x0fPlanningProblem\x12\x1b\n\x13planning_problem_id\x18\x01 \x02(\r\x12(\n\rinitial_state\x18\x02 \x02(\x0b\x32\x11.commonroad.State\x12*\n\x0bgoal_states\x18\x03 \x03(\x0b\x32\x15.commonroad.GoalState'
  ,
  dependencies=[obstacle__pb2.DESCRIPTOR,])




_GOALSTATE = _descriptor.Descriptor(
  name='GoalState',
  full_name='commonroad.GoalState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='commonroad.GoalState.state', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='goal_position_lanelets', full_name='commonroad.GoalState.goal_position_lanelets', index=1,
      number=2, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=54,
  serialized_end=131,
)


_PLANNINGPROBLEM = _descriptor.Descriptor(
  name='PlanningProblem',
  full_name='commonroad.PlanningProblem',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='planning_problem_id', full_name='commonroad.PlanningProblem.planning_problem_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initial_state', full_name='commonroad.PlanningProblem.initial_state', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='goal_states', full_name='commonroad.PlanningProblem.goal_states', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=134,
  serialized_end=266,
)

_GOALSTATE.fields_by_name['state'].message_type = obstacle__pb2._STATE
_PLANNINGPROBLEM.fields_by_name['initial_state'].message_type = obstacle__pb2._STATE
_PLANNINGPROBLEM.fields_by_name['goal_states'].message_type = _GOALSTATE
DESCRIPTOR.message_types_by_name['GoalState'] = _GOALSTATE
DESCRIPTOR.message_types_by_name['PlanningProblem'] = _PLANNINGPROBLEM
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GoalState = _reflection.GeneratedProtocolMessageType('GoalState', (_message.Message,), {
  'DESCRIPTOR' : _GOALSTATE,
  '__module__' : 'planning_problem_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.GoalState)
  })
_sym_db.RegisterMessage(GoalState)

PlanningProblem = _reflection.GeneratedProtocolMessageType('PlanningProblem', (_message.Message,), {
  'DESCRIPTOR' : _PLANNINGPROBLEM,
  '__module__' : 'planning_problem_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.PlanningProblem)
  })
_sym_db.RegisterMessage(PlanningProblem)


# @@protoc_insertion_point(module_scope)
