# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: obstacle.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='obstacle.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0eobstacle.proto\x12\ncommonroad\x1a\nutil.proto\"\x8f\x02\n\x10ObstacleTypeEnum\"\xfa\x01\n\x0cObstacleType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x07\n\x03\x43\x41R\x10\x01\x12\t\n\x05TRUCK\x10\x02\x12\x07\n\x03\x42US\x10\x03\x12\x0b\n\x07\x42ICYCLE\x10\x04\x12\x0e\n\nPEDESTRIAN\x10\x05\x12\x14\n\x10PRIORITY_VEHICLE\x10\x06\x12\x12\n\x0ePARKED_VEHICLE\x10\x07\x12\x15\n\x11\x43ONSTRUCTION_ZONE\x10\x08\x12\t\n\x05TRAIN\x10\t\x12\x11\n\rROAD_BOUNDARY\x10\n\x12\x0e\n\nMOTORCYCLE\x10\x0b\x12\x08\n\x04TAXI\x10\x0c\x12\x0c\n\x08\x42UILDING\x10\r\x12\n\n\x06PILLAR\x10\x0e\x12\x10\n\x0cMEDIAN_STRIP\x10\x0f\"\xd8\x11\n\x05State\x12\"\n\x05point\x18\x01 \x01(\x0b\x32\x11.commonroad.PointH\x00\x12\"\n\x05shape\x18\x02 \x01(\x0b\x32\x11.commonroad.ShapeH\x00\x12\x35\n\x0borientation\x18\x03 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x32\n\x08velocity\x18\x04 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0esteering_angle\x18\x05 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12>\n\x14steering_angle_speed\x18\x06 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x32\n\x08yaw_rate\x18\x07 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nslip_angle\x18\x08 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nroll_angle\x18\t \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\troll_rate\x18\n \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x35\n\x0bpitch_angle\x18\x0b \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\npitch_rate\x18\x0c \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nvelocity_y\x18\r \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nposition_z\x18\x0e \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x34\n\nvelocity_z\x18\x0f \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10roll_angle_front\x18\x10 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0froll_rate_front\x18\x11 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10velocity_y_front\x18\x12 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10position_z_front\x18\x13 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12:\n\x10velocity_z_front\x18\x14 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0froll_angle_rear\x18\x15 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0eroll_rate_rear\x18\x16 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0fvelocity_y_rear\x18\x17 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0fposition_z_rear\x18\x18 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x39\n\x0fvelocity_z_rear\x18\x19 \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x43\n\x19\x66ront_wheel_angular_speed\x18\x1a \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x42\n\x18rear_wheel_angular_speed\x18\x1b \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12H\n\x1eleft_front_wheel_angular_speed\x18\x1c \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12I\n\x1fright_front_wheel_angular_speed\x18\x1d \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12G\n\x1dleft_rear_wheel_angular_speed\x18\x1e \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12H\n\x1eright_rear_wheel_angular_speed\x18\x1f \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\tdelta_y_f\x18  \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\tdelta_y_r\x18! \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x36\n\x0c\x61\x63\x63\x65leration\x18\" \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0e\x61\x63\x63\x65leration_y\x18# \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12.\n\x04jerk\x18$ \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x33\n\tcurvature\x18% \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x38\n\x0e\x63urvature_rate\x18& \x01(\x0b\x32 .commonroad.FloatExactOrInterval\x12\x35\n\ttime_step\x18\' \x02(\x0b\x32\".commonroad.IntegerExactOrIntervalB\n\n\x08position\"\xd8\x01\n\x0bSignalState\x12\x35\n\ttime_step\x18\x01 \x01(\x0b\x32\".commonroad.IntegerExactOrInterval\x12\x0c\n\x04horn\x18\x02 \x01(\x08\x12\x16\n\x0eindicator_left\x18\x03 \x01(\x08\x12\x17\n\x0findicator_right\x18\x04 \x01(\x08\x12\x16\n\x0e\x62raking_lights\x18\x05 \x01(\x08\x12\x1d\n\x15hazard_warning_lights\x18\x06 \x01(\x08\x12\x1c\n\x14\x66lashing_blue_lights\x18\x07 \x01(\x08\"d\n\tOccupancy\x12\x35\n\ttime_step\x18\x01 \x02(\x0b\x32\".commonroad.IntegerExactOrInterval\x12 \n\x05shape\x18\x02 \x02(\x0b\x32\x11.commonroad.Shape\":\n\x0cOccupancySet\x12*\n\x0boccupancies\x18\x01 \x03(\x0b\x32\x15.commonroad.Occupancy\"J\n\nTrajectory\x12\x19\n\x11initial_time_step\x18\x01 \x02(\r\x12!\n\x06states\x18\x02 \x03(\x0b\x32\x11.commonroad.State\"d\n\x14TrajectoryPrediction\x12*\n\ntrajectory\x18\x01 \x02(\x0b\x32\x16.commonroad.Trajectory\x12 \n\x05shape\x18\x02 \x02(\x0b\x32\x11.commonroad.Shape\"`\n\x12SetBasedPrediction\x12\x19\n\x11initial_time_step\x18\x01 \x02(\r\x12/\n\roccupancy_set\x18\x02 \x02(\x0b\x32\x18.commonroad.OccupancySet'
  ,
  dependencies=[util__pb2.DESCRIPTOR,])



_OBSTACLETYPEENUM_OBSTACLETYPE = _descriptor.EnumDescriptor(
  name='ObstacleType',
  full_name='commonroad.ObstacleTypeEnum.ObstacleType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CAR', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRUCK', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUS', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BICYCLE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PEDESTRIAN', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PRIORITY_VEHICLE', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PARKED_VEHICLE', index=7, number=7,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CONSTRUCTION_ZONE', index=8, number=8,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TRAIN', index=9, number=9,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ROAD_BOUNDARY', index=10, number=10,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MOTORCYCLE', index=11, number=11,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='TAXI', index=12, number=12,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BUILDING', index=13, number=13,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PILLAR', index=14, number=14,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='MEDIAN_STRIP', index=15, number=15,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=64,
  serialized_end=314,
)
_sym_db.RegisterEnumDescriptor(_OBSTACLETYPEENUM_OBSTACLETYPE)


_OBSTACLETYPEENUM = _descriptor.Descriptor(
  name='ObstacleTypeEnum',
  full_name='commonroad.ObstacleTypeEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OBSTACLETYPEENUM_OBSTACLETYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=43,
  serialized_end=314,
)


_STATE = _descriptor.Descriptor(
  name='State',
  full_name='commonroad.State',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='point', full_name='commonroad.State.point', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='commonroad.State.shape', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='orientation', full_name='commonroad.State.orientation', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity', full_name='commonroad.State.velocity', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='steering_angle', full_name='commonroad.State.steering_angle', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='steering_angle_speed', full_name='commonroad.State.steering_angle_speed', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='yaw_rate', full_name='commonroad.State.yaw_rate', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='slip_angle', full_name='commonroad.State.slip_angle', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_angle', full_name='commonroad.State.roll_angle', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_rate', full_name='commonroad.State.roll_rate', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pitch_angle', full_name='commonroad.State.pitch_angle', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='pitch_rate', full_name='commonroad.State.pitch_rate', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_y', full_name='commonroad.State.velocity_y', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_z', full_name='commonroad.State.position_z', index=13,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_z', full_name='commonroad.State.velocity_z', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_angle_front', full_name='commonroad.State.roll_angle_front', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_rate_front', full_name='commonroad.State.roll_rate_front', index=16,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_y_front', full_name='commonroad.State.velocity_y_front', index=17,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_z_front', full_name='commonroad.State.position_z_front', index=18,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_z_front', full_name='commonroad.State.velocity_z_front', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_angle_rear', full_name='commonroad.State.roll_angle_rear', index=20,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='roll_rate_rear', full_name='commonroad.State.roll_rate_rear', index=21,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_y_rear', full_name='commonroad.State.velocity_y_rear', index=22,
      number=23, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position_z_rear', full_name='commonroad.State.position_z_rear', index=23,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='velocity_z_rear', full_name='commonroad.State.velocity_z_rear', index=24,
      number=25, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='front_wheel_angular_speed', full_name='commonroad.State.front_wheel_angular_speed', index=25,
      number=26, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rear_wheel_angular_speed', full_name='commonroad.State.rear_wheel_angular_speed', index=26,
      number=27, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left_front_wheel_angular_speed', full_name='commonroad.State.left_front_wheel_angular_speed', index=27,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right_front_wheel_angular_speed', full_name='commonroad.State.right_front_wheel_angular_speed', index=28,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='left_rear_wheel_angular_speed', full_name='commonroad.State.left_rear_wheel_angular_speed', index=29,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='right_rear_wheel_angular_speed', full_name='commonroad.State.right_rear_wheel_angular_speed', index=30,
      number=31, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta_y_f', full_name='commonroad.State.delta_y_f', index=31,
      number=32, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='delta_y_r', full_name='commonroad.State.delta_y_r', index=32,
      number=33, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acceleration', full_name='commonroad.State.acceleration', index=33,
      number=34, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='acceleration_y', full_name='commonroad.State.acceleration_y', index=34,
      number=35, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='jerk', full_name='commonroad.State.jerk', index=35,
      number=36, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='curvature', full_name='commonroad.State.curvature', index=36,
      number=37, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='curvature_rate', full_name='commonroad.State.curvature_rate', index=37,
      number=38, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_step', full_name='commonroad.State.time_step', index=38,
      number=39, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='position', full_name='commonroad.State.position',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=317,
  serialized_end=2581,
)


_SIGNALSTATE = _descriptor.Descriptor(
  name='SignalState',
  full_name='commonroad.SignalState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_step', full_name='commonroad.SignalState.time_step', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='horn', full_name='commonroad.SignalState.horn', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='indicator_left', full_name='commonroad.SignalState.indicator_left', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='indicator_right', full_name='commonroad.SignalState.indicator_right', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='braking_lights', full_name='commonroad.SignalState.braking_lights', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hazard_warning_lights', full_name='commonroad.SignalState.hazard_warning_lights', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flashing_blue_lights', full_name='commonroad.SignalState.flashing_blue_lights', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2584,
  serialized_end=2800,
)


_OCCUPANCY = _descriptor.Descriptor(
  name='Occupancy',
  full_name='commonroad.Occupancy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='time_step', full_name='commonroad.Occupancy.time_step', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='commonroad.Occupancy.shape', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2802,
  serialized_end=2902,
)


_OCCUPANCYSET = _descriptor.Descriptor(
  name='OccupancySet',
  full_name='commonroad.OccupancySet',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='occupancies', full_name='commonroad.OccupancySet.occupancies', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2904,
  serialized_end=2962,
)


_TRAJECTORY = _descriptor.Descriptor(
  name='Trajectory',
  full_name='commonroad.Trajectory',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_time_step', full_name='commonroad.Trajectory.initial_time_step', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='states', full_name='commonroad.Trajectory.states', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2964,
  serialized_end=3038,
)


_TRAJECTORYPREDICTION = _descriptor.Descriptor(
  name='TrajectoryPrediction',
  full_name='commonroad.TrajectoryPrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='trajectory', full_name='commonroad.TrajectoryPrediction.trajectory', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='commonroad.TrajectoryPrediction.shape', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3040,
  serialized_end=3140,
)


_SETBASEDPREDICTION = _descriptor.Descriptor(
  name='SetBasedPrediction',
  full_name='commonroad.SetBasedPrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_time_step', full_name='commonroad.SetBasedPrediction.initial_time_step', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='occupancy_set', full_name='commonroad.SetBasedPrediction.occupancy_set', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3142,
  serialized_end=3238,
)

_OBSTACLETYPEENUM_OBSTACLETYPE.containing_type = _OBSTACLETYPEENUM
_STATE.fields_by_name['point'].message_type = util__pb2._POINT
_STATE.fields_by_name['shape'].message_type = util__pb2._SHAPE
_STATE.fields_by_name['orientation'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['steering_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['steering_angle_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['yaw_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['slip_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['pitch_angle'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['pitch_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_y'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['position_z'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_z'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_angle_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_rate_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_y_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['position_z_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_z_front'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_angle_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['roll_rate_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_y_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['position_z_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['velocity_z_rear'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['front_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['rear_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['left_front_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['right_front_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['left_rear_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['right_rear_wheel_angular_speed'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['delta_y_f'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['delta_y_r'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['acceleration'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['acceleration_y'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['jerk'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['curvature'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['curvature_rate'].message_type = util__pb2._FLOATEXACTORINTERVAL
_STATE.fields_by_name['time_step'].message_type = util__pb2._INTEGEREXACTORINTERVAL
_STATE.oneofs_by_name['position'].fields.append(
  _STATE.fields_by_name['point'])
_STATE.fields_by_name['point'].containing_oneof = _STATE.oneofs_by_name['position']
_STATE.oneofs_by_name['position'].fields.append(
  _STATE.fields_by_name['shape'])
_STATE.fields_by_name['shape'].containing_oneof = _STATE.oneofs_by_name['position']
_SIGNALSTATE.fields_by_name['time_step'].message_type = util__pb2._INTEGEREXACTORINTERVAL
_OCCUPANCY.fields_by_name['time_step'].message_type = util__pb2._INTEGEREXACTORINTERVAL
_OCCUPANCY.fields_by_name['shape'].message_type = util__pb2._SHAPE
_OCCUPANCYSET.fields_by_name['occupancies'].message_type = _OCCUPANCY
_TRAJECTORY.fields_by_name['states'].message_type = _STATE
_TRAJECTORYPREDICTION.fields_by_name['trajectory'].message_type = _TRAJECTORY
_TRAJECTORYPREDICTION.fields_by_name['shape'].message_type = util__pb2._SHAPE
_SETBASEDPREDICTION.fields_by_name['occupancy_set'].message_type = _OCCUPANCYSET
DESCRIPTOR.message_types_by_name['ObstacleTypeEnum'] = _OBSTACLETYPEENUM
DESCRIPTOR.message_types_by_name['State'] = _STATE
DESCRIPTOR.message_types_by_name['SignalState'] = _SIGNALSTATE
DESCRIPTOR.message_types_by_name['Occupancy'] = _OCCUPANCY
DESCRIPTOR.message_types_by_name['OccupancySet'] = _OCCUPANCYSET
DESCRIPTOR.message_types_by_name['Trajectory'] = _TRAJECTORY
DESCRIPTOR.message_types_by_name['TrajectoryPrediction'] = _TRAJECTORYPREDICTION
DESCRIPTOR.message_types_by_name['SetBasedPrediction'] = _SETBASEDPREDICTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ObstacleTypeEnum = _reflection.GeneratedProtocolMessageType('ObstacleTypeEnum', (_message.Message,), {
  'DESCRIPTOR' : _OBSTACLETYPEENUM,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.ObstacleTypeEnum)
  })
_sym_db.RegisterMessage(ObstacleTypeEnum)

State = _reflection.GeneratedProtocolMessageType('State', (_message.Message,), {
  'DESCRIPTOR' : _STATE,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.State)
  })
_sym_db.RegisterMessage(State)

SignalState = _reflection.GeneratedProtocolMessageType('SignalState', (_message.Message,), {
  'DESCRIPTOR' : _SIGNALSTATE,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.SignalState)
  })
_sym_db.RegisterMessage(SignalState)

Occupancy = _reflection.GeneratedProtocolMessageType('Occupancy', (_message.Message,), {
  'DESCRIPTOR' : _OCCUPANCY,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Occupancy)
  })
_sym_db.RegisterMessage(Occupancy)

OccupancySet = _reflection.GeneratedProtocolMessageType('OccupancySet', (_message.Message,), {
  'DESCRIPTOR' : _OCCUPANCYSET,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.OccupancySet)
  })
_sym_db.RegisterMessage(OccupancySet)

Trajectory = _reflection.GeneratedProtocolMessageType('Trajectory', (_message.Message,), {
  'DESCRIPTOR' : _TRAJECTORY,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.Trajectory)
  })
_sym_db.RegisterMessage(Trajectory)

TrajectoryPrediction = _reflection.GeneratedProtocolMessageType('TrajectoryPrediction', (_message.Message,), {
  'DESCRIPTOR' : _TRAJECTORYPREDICTION,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TrajectoryPrediction)
  })
_sym_db.RegisterMessage(TrajectoryPrediction)

SetBasedPrediction = _reflection.GeneratedProtocolMessageType('SetBasedPrediction', (_message.Message,), {
  'DESCRIPTOR' : _SETBASEDPREDICTION,
  '__module__' : 'obstacle_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.SetBasedPrediction)
  })
_sym_db.RegisterMessage(SetBasedPrediction)


# @@protoc_insertion_point(module_scope)
