# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: commonroad.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.scenario_tags_pb2 as scenario__tags__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.location_pb2 as location__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.lanelet_pb2 as lanelet__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.traffic_sign_pb2 as traffic__sign__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.traffic_light_pb2 as traffic__light__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.intersection_pb2 as intersection__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.static_obstacle_pb2 as static__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.dynamic_obstacle_pb2 as dynamic__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.environment_obstacle_pb2 as environment__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.phantom_obstacle_pb2 as phantom__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.planning_problem_pb2 as planning__problem__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='commonroad.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10\x63ommonroad.proto\x12\ncommonroad\x1a\nutil.proto\x1a\x13scenario_tags.proto\x1a\x0elocation.proto\x1a\rlanelet.proto\x1a\x12traffic_sign.proto\x1a\x13traffic_light.proto\x1a\x12intersection.proto\x1a\x15static_obstacle.proto\x1a\x16\x64ynamic_obstacle.proto\x1a\x1a\x65nvironment_obstacle.proto\x1a\x16phantom_obstacle.proto\x1a\x16planning_problem.proto\"\xba\x01\n\x13ScenarioInformation\x12\x1b\n\x13\x63ommon_road_version\x18\x01 \x02(\t\x12\x14\n\x0c\x62\x65nchmark_id\x18\x02 \x02(\t\x12#\n\x04\x64\x61te\x18\x03 \x02(\x0b\x32\x15.commonroad.TimeStamp\x12\x0e\n\x06\x61uthor\x18\x04 \x02(\t\x12\x13\n\x0b\x61\x66\x66iliation\x18\x05 \x02(\t\x12\x0e\n\x06source\x18\x06 \x02(\t\x12\x16\n\x0etime_step_size\x18\x07 \x02(\x01\"\xf3\x04\n\nCommonRoad\x12\x34\n\x0binformation\x18\x01 \x02(\x0b\x32\x1f.commonroad.ScenarioInformation\x12/\n\rscenario_tags\x18\x02 \x02(\x0b\x32\x18.commonroad.ScenarioTags\x12&\n\x08location\x18\x03 \x02(\x0b\x32\x14.commonroad.Location\x12%\n\x08lanelets\x18\x04 \x03(\x0b\x32\x13.commonroad.Lanelet\x12.\n\rtraffic_signs\x18\x05 \x03(\x0b\x32\x17.commonroad.TrafficSign\x12\x30\n\x0etraffic_lights\x18\x06 \x03(\x0b\x32\x18.commonroad.TrafficLight\x12/\n\rintersections\x18\x07 \x03(\x0b\x32\x18.commonroad.Intersection\x12\x34\n\x10static_obstacles\x18\x08 \x03(\x0b\x32\x1a.commonroad.StaticObstacle\x12\x36\n\x11\x64ynamic_obstacles\x18\t \x03(\x0b\x32\x1b.commonroad.DynamicObstacle\x12>\n\x15\x65nvironment_obstacles\x18\n \x03(\x0b\x32\x1f.commonroad.EnvironmentObstacle\x12\x36\n\x11phantom_obstacles\x18\x0b \x03(\x0b\x32\x1b.commonroad.PhantomObstacle\x12\x36\n\x11planning_problems\x18\x0c \x03(\x0b\x32\x1b.commonroad.PlanningProblem'
  ,
  dependencies=[util__pb2.DESCRIPTOR,scenario__tags__pb2.DESCRIPTOR,location__pb2.DESCRIPTOR,lanelet__pb2.DESCRIPTOR,traffic__sign__pb2.DESCRIPTOR,traffic__light__pb2.DESCRIPTOR,intersection__pb2.DESCRIPTOR,static__obstacle__pb2.DESCRIPTOR,dynamic__obstacle__pb2.DESCRIPTOR,environment__obstacle__pb2.DESCRIPTOR,phantom__obstacle__pb2.DESCRIPTOR,planning__problem__pb2.DESCRIPTOR,])




_SCENARIOINFORMATION = _descriptor.Descriptor(
  name='ScenarioInformation',
  full_name='commonroad.ScenarioInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_road_version', full_name='commonroad.ScenarioInformation.common_road_version', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='benchmark_id', full_name='commonroad.ScenarioInformation.benchmark_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='date', full_name='commonroad.ScenarioInformation.date', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='author', full_name='commonroad.ScenarioInformation.author', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='affiliation', full_name='commonroad.ScenarioInformation.affiliation', index=4,
      number=5, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source', full_name='commonroad.ScenarioInformation.source', index=5,
      number=6, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_step_size', full_name='commonroad.ScenarioInformation.time_step_size', index=6,
      number=7, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=281,
  serialized_end=467,
)


_COMMONROAD = _descriptor.Descriptor(
  name='CommonRoad',
  full_name='commonroad.CommonRoad',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='information', full_name='commonroad.CommonRoad.information', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scenario_tags', full_name='commonroad.CommonRoad.scenario_tags', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='commonroad.CommonRoad.location', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lanelets', full_name='commonroad.CommonRoad.lanelets', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_signs', full_name='commonroad.CommonRoad.traffic_signs', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_lights', full_name='commonroad.CommonRoad.traffic_lights', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='intersections', full_name='commonroad.CommonRoad.intersections', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_obstacles', full_name='commonroad.CommonRoad.static_obstacles', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dynamic_obstacles', full_name='commonroad.CommonRoad.dynamic_obstacles', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment_obstacles', full_name='commonroad.CommonRoad.environment_obstacles', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phantom_obstacles', full_name='commonroad.CommonRoad.phantom_obstacles', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='planning_problems', full_name='commonroad.CommonRoad.planning_problems', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=470,
  serialized_end=1097,
)

_SCENARIOINFORMATION.fields_by_name['date'].message_type = util__pb2._TIMESTAMP
_COMMONROAD.fields_by_name['information'].message_type = _SCENARIOINFORMATION
_COMMONROAD.fields_by_name['scenario_tags'].message_type = scenario__tags__pb2._SCENARIOTAGS
_COMMONROAD.fields_by_name['location'].message_type = location__pb2._LOCATION
_COMMONROAD.fields_by_name['lanelets'].message_type = lanelet__pb2._LANELET
_COMMONROAD.fields_by_name['traffic_signs'].message_type = traffic__sign__pb2._TRAFFICSIGN
_COMMONROAD.fields_by_name['traffic_lights'].message_type = traffic__light__pb2._TRAFFICLIGHT
_COMMONROAD.fields_by_name['intersections'].message_type = intersection__pb2._INTERSECTION
_COMMONROAD.fields_by_name['static_obstacles'].message_type = static__obstacle__pb2._STATICOBSTACLE
_COMMONROAD.fields_by_name['dynamic_obstacles'].message_type = dynamic__obstacle__pb2._DYNAMICOBSTACLE
_COMMONROAD.fields_by_name['environment_obstacles'].message_type = environment__obstacle__pb2._ENVIRONMENTOBSTACLE
_COMMONROAD.fields_by_name['phantom_obstacles'].message_type = phantom__obstacle__pb2._PHANTOMOBSTACLE
_COMMONROAD.fields_by_name['planning_problems'].message_type = planning__problem__pb2._PLANNINGPROBLEM
DESCRIPTOR.message_types_by_name['ScenarioInformation'] = _SCENARIOINFORMATION
DESCRIPTOR.message_types_by_name['CommonRoad'] = _COMMONROAD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ScenarioInformation = _reflection.GeneratedProtocolMessageType('ScenarioInformation', (_message.Message,), {
  'DESCRIPTOR' : _SCENARIOINFORMATION,
  '__module__' : 'commonroad_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.ScenarioInformation)
  })
_sym_db.RegisterMessage(ScenarioInformation)

CommonRoad = _reflection.GeneratedProtocolMessageType('CommonRoad', (_message.Message,), {
  'DESCRIPTOR' : _COMMONROAD,
  '__module__' : 'commonroad_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.CommonRoad)
  })
_sym_db.RegisterMessage(CommonRoad)


# @@protoc_insertion_point(module_scope)
